//
// Copyright (C) 2022 Intel Corporation.
// SPDX-License-Identifier: Apache 2.0
//

#pragma once

#include "vpux/compiler/dialect/VPUIP/ops.hpp"
#include "vpux/utils/core/logger.hpp"
#include "vpux/utils/core/small_string.hpp"

namespace vpux {
namespace VPUIP {

// TODO: E60214, need support more sw kernel task type. Currently only enable MVN
const SmallVector<StringLiteral> SW_KERNELS_SUPPORTING_TILING = {"singleShaveMVN",       "singleShaveInterpolate",
                                                                 "swish_fp16",           "gelu_fp16",
                                                                 "singleShaveSoftmax",   "hswish_fp16",
                                                                 "eltwise_mul",          "activation_hardsigmoid",
                                                                 "single_shave_convert", "tanh_fp16",
                                                                 "single_shave_topk",    "single_shave_gather",
                                                                 "sigmoid_fp16",         "single_shave_depth_to_space",
                                                                 "clamp_fp16",           "eltwise_min",
                                                                 "eltwise_max",          "eltwise_power",
                                                                 "activation_abs",       "eltwise_div",
                                                                 "prelu_fp16",           "normalize_l2_fp16"};

const SmallVector<StringLiteral> SW_KERNELS_SUPPORTING_STRIDE = {"singleShaveMVN"};

constexpr StringLiteral SW_KERNEL_NAME_PREFIX = "builtin_";

SmallVector<mlir::Attribute> kernelArgsRange(VPUIP::SwKernelOp swKernelOp);

mlir::SymbolRefAttr createBuiltInFunction(mlir::ModuleOp module, mlir::StringRef builtInFunctionName,
                                          const ArrayRef<mlir::Type> inputTypes, mlir::StringRef kernelEntryName,
                                          mlir::StringRef kernelSourceFileName, const vpux::Logger& log);

void createRuntimeKernelDefinition(mlir::ModuleOp module, const Logger& log);

void initSwKernel(vpux::VPUIP::SwKernelOp swKernelOp, mlir::ValueRange inputs, mlir::ValueRange outputBuffs,
                  mlir::ArrayRef<mlir::Attribute> args, const vpux::Logger& log);

void initSwKernel(VPUIP::SwKernelOp swKernelOp, VPUIP::SwKernelRun swKernelRunOp, const vpux::Logger& log);

SmallString getSwKernelEntryName(VPUIP::SwKernelOp swKernelOp);
mlir::ModuleOp getVPUSWModule(mlir::ModuleOp module, const Logger& log);
bool isSwKernelTilingSupported(VPUIP::SwKernelOp swKernelOp);
bool isStridedDataAccessSupported(VPUIP::SwKernelOp swKernelOp);

InputTiling backInferSwKernelInputTile(VPUIP::SwKernelOp swKernelOp, const vpux::TileInfo& outputTile, Logger log);

SmallVector<mlir::Attribute> getSwkernelNewAttrsAfterTiling(VPUIP::SwKernelOp swKernelOp,
                                                            ArrayRef<mlir::Attribute> origAttr,
                                                            const TilingInfo& inputTiling, const TileInfo& outTile,
                                                            Logger log);

SmallVector<vpux::NDTypeInterface> getSwKernelTiledTypes(VPUIP::SwKernelOp swKernelOp);

bool isCacheHandlingOp(VPUIP::SwKernelOp swKernelOp);
}  // namespace VPUIP
}  // namespace vpux
