//
// Copyright (C) 2022-2023 Intel Corporation.
// SPDX-License-Identifier: Apache 2.0
//

// RUN: vpux-opt --mlir-print-debuginfo --init-compiler="vpu-arch=%arch% allow-custom-values=true" --lower-VPUIP-to-ELF %data_path_npu%/profiling-37XX.mlir.txt | vpux-translate --vpu-arch=%arch% --export-ELF -o %t
// RUN: prof_parser -b %t -p %data_path_npu%/profiling-0-37XX.bin -f json | FileCheck %s
// REQUIRES: arch-VPUX37XX

//CHECK: {"traceEvents":[
//CHECK: {"name": "process_name", "ph": "M", "pid":0, "args": {"name" : "DMA"}},
//CHECK: {"name": "process_sort_index", "ph": "M", "pid":0, "args": {"sort_index" : "0"}},
//CHECK: {"name": "thread_name", "ph": "M", "pid":0, "tid":0, "args": {"name" : "DMA"}},
//CHECK: {"name": "thread_name", "ph": "M", "pid":0, "tid":1, "args": {"name" : "DMA"}},
//CHECK: {"name": "process_name", "ph": "M", "pid":1, "args": {"name" : "Cluster (0)"}},
//CHECK: {"name": "process_sort_index", "ph": "M", "pid":1, "args": {"sort_index" : "1"}},
//CHECK: {"name": "thread_name", "ph": "M", "pid":1, "tid":0, "args": {"name" : "DPU"}},
//CHECK: {"name": "thread_name", "ph": "M", "pid":1, "tid":1, "args": {"name" : "SW / Shave"}},
//CHECK: {"name": "thread_name", "ph": "M", "pid":1, "tid":2, "args": {"name" : "SW / Shave"}},
//CHECK: {"name": "process_name", "ph": "M", "pid":2, "args": {"name" : "Cluster (1)"}},
//CHECK: {"name": "process_sort_index", "ph": "M", "pid":2, "args": {"sort_index" : "2"}},
//CHECK: {"name": "thread_name", "ph": "M", "pid":2, "tid":0, "args": {"name" : "DPU"}},
//CHECK: {"name": "thread_name", "ph": "M", "pid":2, "tid":1, "args": {"name" : "SW / Shave"}},
//CHECK: {"name": "thread_name", "ph": "M", "pid":2, "tid":2, "args": {"name" : "SW / Shave"}},
//CHECK: {"name": "process_name", "ph": "M", "pid":3, "args": {"name" : "Layers"}},
//CHECK: {"name": "process_sort_index", "ph": "M", "pid":3, "args": {"sort_index" : "3"}},
//CHECK: {"name": "thread_name", "ph": "M", "pid":3, "tid":0, "args": {"name" : "Layers"}},
//CHECK: {"name": "thread_name", "ph": "M", "pid":3, "tid":1, "args": {"name" : "Layers"}},
//CHECK: {"name":"conv1/WithoutBiases?t_Convolution", "cat":"DMA", "ph":"X", "ts":0.000, "dur":2.005, "pid":0, "tid":0},
//CHECK: {"name":"conv1/WithoutBiases?t_Convolution/_expand_copy_3_2", "cat":"DMA", "ph":"X", "ts":4.089, "dur":1.562, "pid":0, "tid":0},
//CHECK: {"name":"conv1/WithoutBiases?t_Convolution", "cat":"DMA", "ph":"X", "ts":19.870, "dur":1.171, "pid":0, "tid":0},
//CHECK: {"name":"conv1/WithoutBiases?t_Convolution/_cluster_1", "cat":"DMA", "ph":"X", "ts":21.276, "dur":0.859, "pid":0, "tid":0},
//CHECK: {"name":"conv1/WithoutBiases?t_Convolution/_cluster_0", "cat":"DMA", "ph":"X", "ts":21.432, "dur":0.885, "pid":0, "tid":1},
//CHECK: {"name":"conv1/WithoutBiases?t_Convolution/_fused_constant/_fused_tile", "cat":"DMA", "ph":"X", "ts":22.552, "dur":0.703, "pid":0, "tid":0},
//CHECK: {"name":"conv1/WithoutBiases?t_Convolution/_cluster_1", "cat":"DMA", "ph":"X", "ts":23.464, "dur":2.760, "pid":0, "tid":0},
//CHECK: {"name":"conv1/WithoutBiases?t_Convolution/_cluster_0", "cat":"DMA", "ph":"X", "ts":23.620, "dur":2.760, "pid":0, "tid":1},
//CHECK: {"name":"conv1/WithoutBiases?t_Convolution/_cluster_1", "cat":"DMA", "ph":"X", "ts":26.615, "dur":2.604, "pid":0, "tid":0},
//CHECK: {"name":"conv1/WithoutBiases?t_Convolution/_cluster_0", "cat":"DMA", "ph":"X", "ts":26.771, "dur":2.760, "pid":0, "tid":1},
//CHECK: {"name":"conv2/WithoutBiases?t_Convolution/_fused_constant/_fused_tile", "cat":"DMA", "ph":"X", "ts":29.766, "dur":2.317, "pid":0, "tid":0},
//CHECK: {"name":"relu2?t_Relu/_cluster_1", "cat":"DMA", "ph":"X", "ts":368.828, "dur":1.119, "pid":0, "tid":0},
//CHECK: {"name":"relu2?t_Relu/_cluster_0", "cat":"DMA", "ph":"X", "ts":368.984, "dur":0.807, "pid":0, "tid":1},
//CHECK: {"name":"relu2?t_Relu/_cluster_0", "cat":"DMA", "ph":"X", "ts":370.104, "dur":0.625, "pid":0, "tid":0},
//CHECK: {"name":"relu2?t_Relu/_cluster_1", "cat":"DMA", "ph":"X", "ts":370.312, "dur":0.703, "pid":0, "tid":1},
//CHECK: {"name":"relu2?t_Relu/_cluster_0", "cat":"DMA", "ph":"X", "ts":371.250, "dur":0.755, "pid":0, "tid":0},
//CHECK: {"name":"relu2?t_Relu/_cluster_1", "cat":"DMA", "ph":"X", "ts":371.406, "dur":0.755, "pid":0, "tid":1},
//CHECK: {"name":"relu2?t_Relu/_cluster_0", "cat":"DMA", "ph":"X", "ts":372.318, "dur":0.807, "pid":0, "tid":0},
//CHECK: {"name":"relu2?t_Relu/_cluster_1", "cat":"DMA", "ph":"X", "ts":372.474, "dur":0.807, "pid":0, "tid":1},
//CHECK: {"name":"output?t_Output/_cluster_0", "cat":"DMA", "ph":"X", "ts":379.349, "dur":0.911, "pid":0, "tid":0},
//CHECK: {"name":"output?t_Output/_cluster_1", "cat":"DMA", "ph":"X", "ts":379.505, "dur":0.911, "pid":0, "tid":1},
//CHECK: {"name":"output?t_Output/_cluster_0", "cat":"DMA", "ph":"X", "ts":380.573, "dur":0.625, "pid":0, "tid":0},
//CHECK: {"name":"output?t_Output/_cluster_1", "cat":"DMA", "ph":"X", "ts":380.729, "dur":0.625, "pid":0, "tid":1},
//CHECK: {"name":"conv1/WithoutBiases?t_Convolution/cluster_0", "cat":"DPU", "ph":"X", "ts":22.350, "dur":0.661, "pid":1, "tid":0},
//CHECK: {"name":"conv1/WithoutBiases?t_Convolution/Duplicated_2/cluster_0", "cat":"DPU", "ph":"X", "ts":29.531, "dur":32.481, "pid":1, "tid":0},
//CHECK: {"name":"relu1?t_Relu/cluster_0", "cat":"DPU", "ph":"X", "ts":63.683, "dur":10.073, "pid":1, "tid":0},
//CHECK: {"name":"conv2/WithoutBiases?t_Convolution/cluster_0", "cat":"DPU", "ph":"X", "ts":74.377, "dur":285.976, "pid":1, "tid":0},
//CHECK: {"name":"relu2?t_Relu/cluster_0", "cat":"DPU", "ph":"X", "ts":363.194, "dur":4.346, "pid":1, "tid":0},
//CHECK: {"name":"conv1/WithoutBiases?t_Convolution/tile_0/cluster_0", "cat":"SW", "ph":"X", "ts":9.896, "dur":9.453, "pid":1, "tid":1},
//CHECK: {"name":"conv1/WithoutBiases?t_Convolution/tile_1/cluster_0", "cat":"SW", "ph":"X", "ts":10.026, "dur":7.343, "pid":1, "tid":2},
//CHECK: {"name":"output?t_Output/tile_1/cluster_0", "cat":"SW", "ph":"X", "ts":373.932, "dur":4.505, "pid":1, "tid":1},
//CHECK: {"name":"output?t_Output/tile_0/cluster_0", "cat":"SW", "ph":"X", "ts":374.062, "dur":4.635, "pid":1, "tid":2},
//CHECK: {"name":"conv1/WithoutBiases?t_Convolution/cluster_1", "cat":"DPU", "ph":"X", "ts":22.376, "dur":0.666, "pid":2, "tid":0},
//CHECK: {"name":"conv1/WithoutBiases?t_Convolution/Duplicated_2/cluster_1", "cat":"DPU", "ph":"X", "ts":29.595, "dur":33.103, "pid":2, "tid":0},
//CHECK: {"name":"relu1?t_Relu/cluster_1", "cat":"DPU", "ph":"X", "ts":63.944, "dur":6.707, "pid":2, "tid":0},
//CHECK: {"name":"conv2/WithoutBiases?t_Convolution/cluster_1", "cat":"DPU", "ph":"X", "ts":74.001, "dur":289.007, "pid":2, "tid":0},
//CHECK: {"name":"relu2?t_Relu/cluster_1", "cat":"DPU", "ph":"X", "ts":363.652, "dur":3.536, "pid":2, "tid":0},
//CHECK: {"name":"output?t_Output/tile_1/cluster_1", "cat":"SW", "ph":"X", "ts":373.672, "dur":4.895, "pid":2, "tid":1},
//CHECK: {"name":"output?t_Output/tile_0/cluster_1", "cat":"SW", "ph":"X", "ts":373.802, "dur":5.026, "pid":2, "tid":2},
//CHECK: {"name":"conv1/WithoutBiases", "cat":"Layer", "ph":"X", "ts":0.000, "dur":62.698, "pid":3, "tid":0, "args":{"Layer type": "Convolution"}},
//CHECK: {"name":"conv2/WithoutBiases", "cat":"Layer", "ph":"X", "ts":29.766, "dur":333.242, "pid":3, "tid":1, "args":{"Layer type": "Convolution"}},
//CHECK: {"name":"relu1", "cat":"Layer", "ph":"X", "ts":63.683, "dur":10.073, "pid":3, "tid":0, "args":{"Layer type": "Relu"}},
//CHECK: {"name":"relu2", "cat":"Layer", "ph":"X", "ts":363.194, "dur":10.087, "pid":3, "tid":0, "args":{"Layer type": "Relu"}},
//CHECK: {"name":"output", "cat":"Layer", "ph":"X", "ts":373.672, "dur":7.682, "pid":3, "tid":0, "args":{"Layer type": "Output"}}
//CHECK: ],
//CHECK: "taskStatistics": {
//CHECK: "total duration":381.354,
//CHECK: "DMA duration":20.383,
//CHECK: "DPU duration":337.285,
//CHECK: "SW duration":14.609,
//CHECK: "DMA-DPU overlap":2.807,
//CHECK: "DMA-SW overlap":0.000,
//CHECK: "SW-DPU overlap":0.000,
//CHECK: "all tasks union":369.470,
//CHECK: "total idle":11.884,
//CHECK: "SW duration without DPU overlap":14.609,
//CHECK: "DMA duration without overlaps":17.576,
//CHECK: "Sum of DMA task durations":29.836,
//CHECK: "Sum of DPU task durations":666.556,
//CHECK: "Sum of SW task durations":35.857
//CHECK: },
//CHECK: "displayTimeUnit": "ns"
//CHECK: }
